# -*- coding: utf-8 -*-

import grokcore.component as grok
from zeam.form.base.interfaces import IFormCanvas
from dolmen.forms.crud import IFieldsCustomization


class FieldsCustomizer(grok.MultiAdapter):
    grok.baseclass()
    grok.adapts(None, IFormCanvas, None)
    grok.implements(IFieldsCustomization)

    def __init__(self, context, form, request):
        self.context = context
        self.form = form
        self.request = request

    def __call__(self, fields):
        raise NotImplementedError("""Implement your own.""")



# -*- coding: utf-8 -*-

import zope.schema
import megrok.layout
import dolmen.content
import grokcore.component as grok
from zope.interface import Interface
from zope.container.constraints import contains


class IDesertWarrior(dolmen.content.IBaseContent):
    """Defines a warrior living in the desert.
    """
    water = zope.schema.Int(
        title=u"Number water gallons owned",
        default=1,
        required=True,
        )


class IDesertCave(dolmen.content.IBaseContent):
    """Defines a cave carved in the mountains by the Coriolis storms.
    """
    contains(IDesertWarrior)


class Sietch(dolmen.content.Container):
    """A grotto located on Arrakis.
    """
    dolmen.content.name('sietch')
    dolmen.content.schema(IDesertCave)
    dolmen.content.nofactory()


class TrainingCamp(dolmen.content.Factory):
    """A camp that produces Fremen warriors.
    """
    dolmen.content.name(u"fremen")
    title = u"Fremen training camp"


class Fremen(dolmen.content.Content):
    """Inhabitants on the deep deserts. They live in sietches.
    """
    dolmen.content.name(u'Fremen Warrior')
    dolmen.content.schema(IDesertWarrior)
    dolmen.content.factory(TrainingCamp)
    dolmen.content.require('zope.ManageContent')


class Harkonnen(dolmen.content.Content):
    """A native of Giedi Prime.
    """
    dolmen.content.name('Harkonnen')
    dolmen.content.nofactory()


class MyLayout(megrok.layout.Layout):
    grok.context(Interface)

    def render(self):
        return self.view.content()



# -*- coding: utf-8 -*-

import dolmen.content
import grokcore.component as grok

from dolmen.forms.crud import IFactoryAdding
from zope.component import queryMultiAdapter, queryUtility
from zope.container.constraints import checkObject
from zope.container.interfaces import IContainer, INameChooser
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.http import IHTTPRequest
from zope.schema.fieldproperty import FieldProperty
from zope.security.interfaces import Unauthorized
from zope.security.management import checkPermission
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.traversing.interfaces import ITraversable


class Adder(grok.MultiAdapter):
    grok.name('add')
    grok.adapts(IContainer, IHTTPRequest)
    grok.implements(IFactoryAdding)
    grok.provides(ITraversable)

    __name__ = u""
    factory = FieldProperty(IFactoryAdding['factory'])

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.__parent__ = context

    def traverse(self, name, ignore):
        """See dolmen.content.interfaces.IFactory
        """
        factory = queryUtility(dolmen.content.IFactory, name)

        if factory is not None:
            permission = dolmen.content.require.bind().get(factory.factory)
            if checkPermission(permission, self.context):
                self.factory = factory
                addform = queryMultiAdapter(
                    (self, self.request), name=factory.addform)
                if addform is not None:
                    return addform
            else:
                raise Unauthorized("%r requires the %r permission." %
                                   (factory.factory, permission))
        raise NotFound(self.context, name, self.request)

    def add(self, content):
        """See dolmen.forms.crud.interfaces.IAdding
        """
        container = self.context

        # check precondition
        checkObject(container, '__temporary__', content)

        # choose name in container
        chooser = INameChooser(container)
        name = chooser.chooseName('', content)

        # assign object and returns it
        container[name] = content
        return container.get(name, None)

    def nextURL(self):
        """See dolmen.forms.crud.interfaces.IAdding
        """
        return absoluteURL(self.context, self.request)
